@@ -20,6 +20,7 @@ import java.io.OutputStream;
 import java.io.Serializable;
 import java.net.URL;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Vector;
 import java.util.concurrent.ConcurrentHashMap;
@@ -73,7 +74,7 @@ import java.util.concurrent.Exchanger;
  * the construction of the stack will be aborted.
  *
  * @author Bela Ban
- * @version $Id: JChannel.java,v 1.158.2.1 2007/11/26 21:16:45 vlada Exp $
+ * @version $Id: JChannel.java,v 1.158.2.14 2008/02/25 12:50:35 belaban Exp $
  */
 public class JChannel extends Channel {
 
@@ -122,7 +123,7 @@ public class JChannel extends Channel {
      *if this is set to false, the JChannel will not receive messages sent by itself*/
     private boolean receive_local_msgs=true;
     /*flag to indicate whether the channel will reconnect (reopen) when the exit message is received*/
-    private boolean auto_reconnect=false;
+    private boolean auto_reconnect=true;
     /*flag t indicate whether the state is supposed to be retrieved after the channel is reconnected
      *setting this to true, automatically forces auto_reconnect to true*/
     private boolean auto_getstate=false;
@@ -244,7 +245,7 @@ public class JChannel extends Channel {
      * @throws ChannelException if problems occur during the initialization of
      *                          the protocol stack.
      */
-    protected JChannel(ProtocolStackConfigurator configurator) throws ChannelException {
+    public JChannel(ProtocolStackConfigurator configurator) throws ChannelException {
         init(configurator);
     }
 
@@ -319,23 +320,28 @@ public class JChannel extends Channel {
     public long getSentBytes() {return sent_bytes;}
     public long getReceivedMessages() {return received_msgs;}
     public long getReceivedBytes() {return received_bytes;}
-    public int  getNumberOfTasksInTimer() {return prot_stack != null ? prot_stack.timer.size() : -1;}
-
+    public int getNumberOfTasksInTimer() {
+        ProtocolStack ps=getProtocolStack();
+        return ps != null? ps.timer.size() : -1;
+    }
 
     public int getTimerThreads() {
-        return prot_stack != null? prot_stack.getTimerThreads() : -1;
+        ProtocolStack ps=getProtocolStack();
+        return ps != null? ps.getTimerThreads() : -1;
     }
 
     public String dumpTimerQueue() {
-        return prot_stack != null? prot_stack.dumpTimerQueue() : "<n/a";
+        ProtocolStack ps=getProtocolStack();
+        return ps != null? ps.dumpTimerQueue() : "<n/a";
     }
 
     /**
-     * Returns a pretty-printed form of all the protocols. If include_properties is set,
-     * the properties for each protocol will also be printed.
+     * Returns a pretty-printed form of all the protocols. If include_properties
+     * is set, the properties for each protocol will also be printed.
      */
     public String printProtocolSpec(boolean include_properties) {
-        return prot_stack != null ? prot_stack.printProtocolSpec(include_properties) : null;
+        ProtocolStack ps=getProtocolStack();
+        return ps != null? ps.printProtocolSpec(include_properties) : null;
     }
 
 
@@ -356,20 +362,18 @@ public class JChannel extends Channel {
         startStack(cluster_name);
 
         // only connect if we are not a unicast channel
-        if(cluster_name != null) {
-            if(flush_supported)
-               flush_unblock_promise.reset();
+        if(cluster_name != null) {            
             
             Event connect_event=new Event(Event.CONNECT, cluster_name);
             Object res=downcall(connect_event);  // waits forever until connected (or channel is closed)
             if(res != null && res instanceof Exception) { // the JOIN was rejected by the coordinator
+                stopStack(true, false);
+                init();
                 throw new ChannelException("connect() failed", (Throwable)res);
             }
 
-            //if FLUSH is used do not return from connect() until UNBLOCK event is received
-            boolean singletonMember=my_view != null && my_view.size() == 1;
-            boolean shouldWaitForUnblock=flush_supported && receive_blocks && !singletonMember && !flush_unblock_promise.hasResult();
-            if(shouldWaitForUnblock) {
+            //if FLUSH is used do not return from connect() until UNBLOCK event is received                       
+            if(flushSupported()) {
                try {
                   flush_unblock_promise.getResultWithTimeout(FLUSH_UNBLOCK_TIMEOUT);
                }
@@ -430,6 +434,8 @@ public class JChannel extends Channel {
                 // closed)
                 joinSuccessful=!(res != null && res instanceof Exception);
                 if(!joinSuccessful) {
+                    stopStack(true, false);
+                    init();
                     throw new ChannelException("connect() failed", (Throwable)res);
                 }
 
@@ -459,7 +465,7 @@ public class JChannel extends Channel {
 
             }
             finally {
-                if(flush_supported && canFetchState)
+                if(flushSupported() && canFetchState)
                     stopFlush();
             }
         }
@@ -488,14 +494,8 @@ public class JChannel extends Channel {
                 Event disconnect_event=new Event(Event.DISCONNECT, local_addr);
                 down(disconnect_event);   // DISCONNECT is handled by each layer
             }
-
             connected=false;
-            try {
-                prot_stack.stopStack(); // calls stop() in all protocols, from top to bottom
-            }
-            catch(Exception e) {
-                if(log.isErrorEnabled()) log.error("exception: " + e);
-            }
+            stopStack(true, false);            
             notifyChannelDisconnected(this);
             init(); // sets local_addr=null; changed March 18 2003 (bela) -- prevented successful rejoining
         }
@@ -519,7 +519,7 @@ public class JChannel extends Channel {
     }
 
     /**
-     * Opens the channel.
+     * Opens the channel. Note that the channel is only open, but <em>not connected</em>.
      * This does the following actions:
      * <ol>
      * <li> Resets the receiver queue by calling Queue.reset
@@ -573,18 +573,18 @@ public class JChannel extends Channel {
      * @return Map<String,Map>. A map where the keys are the protocols ("channel" pseudo key is
      * used for the channel itself") and the values are property maps.
      */
-    public Map dumpStats() {
-        Map retval=prot_stack.dumpStats();
+    public Map<String,Object> dumpStats() {
+        Map<String,Object> retval=prot_stack.dumpStats();
         if(retval != null) {
-            Map tmp=dumpChannelStats();
+            Map<String,Long> tmp=dumpChannelStats();
             if(tmp != null)
                 retval.put("channel", tmp);
         }
         return retval;
     }
 
-    private Map dumpChannelStats() {
-        Map retval=new HashMap();
+    protected Map<String,Long> dumpChannelStats() {
+        Map<String,Long> retval=new HashMap<String,Long>();
         retval.put("sent_msgs", new Long(sent_msgs));
         retval.put("sent_bytes", new Long(sent_bytes));
         retval.put("received_msgs", new Long(received_msgs));
@@ -653,12 +653,6 @@ public class JChannel extends Channel {
             Event evt=(timeout <= 0)? (Event)mq.remove() : (Event)mq.remove(timeout);
             Object retval=getEvent(evt);
             evt=null;
-            if(stats) {
-                if(retval != null && retval instanceof Message) {
-                    received_msgs++;
-                    received_bytes+=((Message)retval).getLength();
-                }
-            }
             return retval;
         }
         catch(QueueClosedException queue_closed) {
@@ -930,7 +924,7 @@ public class JChannel extends Channel {
 
         
         StateTransferInfo state_info=new StateTransferInfo(target, state_id, timeout);
-        boolean initiateFlush = flush_supported && useFlushIfPresent;
+        boolean initiateFlush = flushSupported() && useFlushIfPresent;
         
         if(initiateFlush)
             startFlush(false);
@@ -1017,6 +1011,11 @@ public class JChannel extends Channel {
 
         case Event.MSG:
             msg=(Message)evt.getArg();
+            if(stats) {
+                received_msgs++;
+                received_bytes+=msg.getLength();
+            }
+
             if(!receive_local_msgs) {  // discard local messages (sent by myself to me)
                 if(local_addr != null && msg.getSrc() != null)
                     if(local_addr.equals(msg.getSrc()))
@@ -1062,31 +1061,35 @@ public class JChannel extends Channel {
            break;    
 
         case Event.GET_STATE_OK:
-            StateTransferInfo state_info=(StateTransferInfo)evt.getArg();
-            byte[] state=state_info.state;
+            StateTransferInfo state_info = (StateTransferInfo) evt.getArg();
+            byte[] state = state_info.state;
 
-            state_promise.setResult(state != null? Boolean.TRUE : Boolean.FALSE);
-            if(up_handler != null) {
-                return up_handler.up(evt);
-            }
+            try{
+                if(up_handler != null){
+                    return up_handler.up(evt);
+                }
 
-            if(state != null) {
-                String state_id=state_info.state_id;
-                if(receiver != null) {
-                    try {
-                        if(receiver instanceof ExtendedReceiver && state_id!=null)
-                            ((ExtendedReceiver)receiver).setState(state_id, state);
-                        else
-                            receiver.setState(state);
-                    }
-                    catch(Throwable t) {
-                        if(log.isWarnEnabled())
-                            log.warn("failed calling setState() in receiver", t);
+                if(state != null){
+                    String state_id = state_info.state_id;
+                    if(receiver != null){
+                        try{
+                            if(receiver instanceof ExtendedReceiver && state_id != null)
+                                ((ExtendedReceiver) receiver).setState(state_id, state);
+                            else
+                                receiver.setState(state);
+                        }catch(Throwable t){
+                            if(log.isWarnEnabled())
+                                log.warn("failed calling setState() in receiver", t);
+                        }
+                    }else{
+                        try{
+                            mq.add(new Event(Event.STATE_RECEIVED, state_info));
+                        }catch(Exception e){
+                        }
                     }
                 }
-                else {
-                    try {mq.add(new Event(Event.STATE_RECEIVED, state_info));} catch(Exception e) {}
-                }
+            }finally{
+                state_promise.setResult(state != null ? Boolean.TRUE : Boolean.FALSE);
             }
             break;
         case Event.STATE_TRANSFER_INPUTSTREAM_CLOSED:
@@ -1227,13 +1230,7 @@ public class JChannel extends Channel {
                         catch(Throwable t) {
                             if(log.isWarnEnabled())
                                 log.warn("failed calling getState() in receiver", t);
-                        }
-                        //Vladimir Oct 29,2007 JChannel.java 1.156
-                        //STREAMING_STATE_TRANSFER does not require return value.
-                        //If there is not return value STATE_TRANSFER_OUTPUTSTREAM event 
-                        //can be processed concurrently by Multiplexer along with other messages 
-                        //for a specific MuxChannel
-                        //@see Multiplexer#passToMuxChannel();
+                        }                       
                     }                    
                 }
                 else if(receiver instanceof Receiver){
@@ -1262,21 +1259,19 @@ public class JChannel extends Channel {
                 }
                 break;
             case Event.UNBLOCK:
-                //discard if client has not set 'receiving blocks' to 'on'
-                if(!receive_blocks) {
-                    return null;
-                }
-                if(receiver instanceof ExtendedReceiver) {
+                //invoke receiver if block receiving is on
+                if(receive_blocks && receiver instanceof ExtendedReceiver) {                                                     
                     try {
                         ((ExtendedReceiver)receiver).unblock();
                     }
                     catch(Throwable t) {
                         if(log.isErrorEnabled())
                             log.error("failed calling unblock() in receiver", t);
-                    }                                        
+                    }                                                            
                 }
-                flush_unblock_promise.setResult(Boolean.TRUE);
-                break;
+                //flip promise
+                flush_unblock_promise.setResult(Boolean.TRUE);              
+                return null;                
             default:
                 break;
         }
@@ -1400,7 +1395,7 @@ public class JChannel extends Channel {
             prot_stack.setup(); // Setup protocol stack (creates protocol, calls init() on them)
         }
         catch(Throwable e) {
-            throw new ChannelException("unable to setup the protocol stack", e);
+            throw new ChannelException("unable to setup the protocol stack: " + e.getMessage(), e);
         }
     }
 
@@ -1439,7 +1434,7 @@ public class JChannel extends Channel {
             this.cluster_name=cluster_name;
 
         try {
-            prot_stack.startStack(); // calls start() in all protocols, from top to bottom
+            prot_stack.startStack(cluster_name); // calls start() in all protocols, from top to bottom
         }
         catch(Throwable e) {
             throw new ChannelException("failed to start protocol stack", e);
@@ -1457,7 +1452,7 @@ public class JChannel extends Channel {
         }
 
         /*create a temporary view, assume this channel is the only member and is the coordinator*/
-        Vector t=new Vector(1);
+        Vector<Address> t=new Vector<Address>(1);
         t.addElement(local_addr);
         my_view=new View(local_addr, 0, t);  // create a dummy view
     }
@@ -1554,31 +1549,32 @@ public class JChannel extends Channel {
         if(disconnect)
             disconnect();                     // leave group if connected
 
-        if(close_mq) {
-            try {
-                if(mq != null)
-                    mq.close(false);              // closes and removes all messages
-            }
-            catch(Exception e) {
-                if(log.isErrorEnabled()) log.error("exception: " + e);
-            }
-        }
+        if(close_mq) 
+            closeMessageQueue(false);       
 
-        if(prot_stack != null) {
-            try {
-                prot_stack.stopStack();
-                prot_stack.destroy();
-            }
-            catch(Exception e) {
-                if(log.isErrorEnabled()) log.error("failed destroying the protocol stack", e);
-            }
-        }
+        stopStack(true, true);
         closed=true;
         connected=false;
         notifyChannelClosed(this);
         init(); // sets local_addr=null; changed March 18 2003 (bela) -- prevented successful rejoining
     }
 
+    protected void stopStack(boolean disconnect, boolean destroy) {
+        if(prot_stack != null) {
+            try {
+                if(disconnect)
+                    prot_stack.stopStack(cluster_name);
+
+                if(destroy)
+                    prot_stack.destroy();
+            }
+            catch(Exception e) {
+                if(log.isErrorEnabled())
+                    log.error("failed destroying the protocol stack", e);
+            }
+        }
+    }
+
 
     public final void closeMessageQueue(boolean flush_entries) {
         if(mq != null)
@@ -1621,7 +1617,7 @@ public class JChannel extends Channel {
      * @return true if FLUSH completed within the timeout
      */
     public boolean startFlush(boolean automatic_resume) {
-        if(!flush_supported) {
+        if(!flushSupported()) {
             throw new IllegalStateException("Flush is not supported, add pbcast.FLUSH protocol to your configuration");
         }           	  
         boolean successfulFlush = (Boolean) downcall(new Event(Event.SUSPEND));
@@ -1632,6 +1628,41 @@ public class JChannel extends Channel {
         return successfulFlush;              
     }
     
+    /**
+     * Performs a partial flush in a cluster for flush participants. 
+     * <p>
+     * All pending messages are flushed out only for flush participants.
+     * Remaining members in a cluster are not included in flush.
+     * Flush participants should be a proper subset of a current view. 
+     * 
+     * <p>
+     * In case of flush collisions, random sleep time backoff algorithm is employed and
+     * flush is reattempted for numberOfAttempts. Therefore this method is guaranteed 
+     * to return after timeout x numberOfAttempts miliseconds.
+     * 
+     * @param automatic_resume Call {@link #stopFlush()} after the flush
+     * @return true if FLUSH completed within the timeout
+     */
+    public boolean startFlush(List<Address> flushParticipants,boolean automatic_resume) {
+        boolean successfulFlush = false;
+        if(!flushSupported()){
+            throw new IllegalStateException("Flush is not supported, add pbcast.FLUSH protocol to your configuration");
+        }
+        View v = getView();
+        if(v != null && v.getMembers().containsAll(flushParticipants)){
+            successfulFlush = (Boolean) downcall(new Event(Event.SUSPEND, flushParticipants));
+        }else{
+            throw new IllegalArgumentException("Current view " + v
+                                               + " does not contain all flush participants "
+                                               + flushParticipants);
+        }
+        
+        if(automatic_resume)
+            stopFlush(flushParticipants);
+
+        return successfulFlush;              
+    }
+    
     /**
      * Will perform a flush of the system, ie. all pending messages are flushed out of the 
      * system and all members ack their reception. After this call returns, no member will
@@ -1649,21 +1680,34 @@ public class JChannel extends Channel {
     }
 
     public void stopFlush() {
-        if(!flush_supported) {
+        if(!flushSupported()) {
+            throw new IllegalStateException("Flush is not supported, add pbcast.FLUSH protocol to your configuration");
+        }
+       
+        down(new Event(Event.RESUME));
+
+        //do not return until UNBLOCK event is received            
+        try {
+            flush_unblock_promise.getResultWithTimeout(FLUSH_UNBLOCK_TIMEOUT);
+        }
+        catch(TimeoutException te) {
+            log.warn("Timeout waiting for UNBLOCK event at " + getLocalAddress());
+        }
+    }
+
+    public void stopFlush(List<Address> flushParticipants) {
+        if(!flushSupported()) {
             throw new IllegalStateException("Flush is not supported, add pbcast.FLUSH protocol to your configuration");
         }
         
-        flush_unblock_promise.reset();
-        down(new Event(Event.RESUME));
-        
-        //do not return until UNBLOCK event is received        
-        boolean shouldWaitForUnblock = receive_blocks;        
-        if(shouldWaitForUnblock){
-           try{              
-              flush_unblock_promise.getResultWithTimeout(FLUSH_UNBLOCK_TIMEOUT);
-           }
-           catch (TimeoutException te){              
-           }
+        down(new Event(Event.RESUME, flushParticipants));
+
+        // do not return until UNBLOCK event is received
+        try {
+            flush_unblock_promise.getResultWithTimeout(FLUSH_UNBLOCK_TIMEOUT);
+        }
+        catch(TimeoutException te) {
+            log.warn("Timeout waiting for UNBLOCK event at " + getLocalAddress());
         }
     }
     
@@ -1672,8 +1716,13 @@ public class JChannel extends Channel {
        return new HashMap<String, Object>(info);
     }
 
+    public void setInfo(String key, Object value) {
+        if(key != null)
+            info.put(key, value);
+    }
+
     Address determineCoordinator() {
-        Vector mbrs=my_view != null? my_view.getMembers() : null;
+        Vector<Address> mbrs=my_view != null? my_view.getMembers() : null;
         if(mbrs == null)
             return null;
         if(!mbrs.isEmpty())
