@@ -383,8 +383,8 @@ public abstract class BasicConnectionTable {
            if(sock_addr != null)
                return sock_addr;
            if(sock != null) {
-               StringBuffer sb;
-               sb=new StringBuffer();
+               StringBuilder sb;
+               sb=new StringBuilder();
                sb.append(sock.getLocalAddress().getHostAddress()).append(':').append(sock.getLocalPort());
                sb.append(" - ").append(sock.getInetAddress().getHostAddress()).append(':').append(sock.getPort());
                sock_addr=sb.toString();
@@ -522,7 +522,7 @@ public abstract class BasicConnectionTable {
                Thread.currentThread().interrupt(); // set interrupt flag again
            }
            catch(Throwable ex) {
-               if(log.isErrorEnabled()) log.error("exception is " + ex);
+               if(log.isErrorEnabled()) log.error("failed sending data to " + peer_addr + ": " + ex);
            }
            finally {
                if(acquire_lock)
@@ -578,7 +578,7 @@ public abstract class BasicConnectionTable {
 
                    if(Version.isBinaryCompatible(version) == false) {
                        if(log.isWarnEnabled())
-                           log.warn(new StringBuffer("packet from ").append(client_addr).append(':').append(client_port).
+                           log.warn(new StringBuilder("packet from ").append(client_addr).append(':').append(client_port).
                                    append(" has different version (").append(Version.print(version)).append(") from ours (").
                                    append(Version.printVersion()).append("). This may cause problems"));
                    }
