@@ -42,7 +42,7 @@ import java.util.concurrent.TimeUnit;
  * to do so.<p>
  * <b>Requirements</b>: lossless delivery, e.g. acknowledgment-based message confirmation.
  * @author Bela Ban
- * @version $Id: GroupRequest.java,v 1.30 2007/07/30 10:53:23 belaban Exp $
+ * @version $Id: GroupRequest.java,v 1.30.2.1 2008/01/22 10:00:59 belaban Exp $
  */
 public class GroupRequest implements RspCollector, Command {
     /** return only first response */
@@ -285,7 +285,7 @@ public class GroupRequest implements RspCollector, Command {
                     rsp.setValue(response_value);
                     rsp.setReceived(true);
                     if(log.isTraceEnabled())
-                        log.trace(new StringBuffer("received response for request ").append(req_id).append(", sender=").
+                        log.trace(new StringBuilder("received response for request ").append(req_id).append(", sender=").
                                 append(sender).append(", val=").append(response_value));
                     if(rsp_filter != null && !rsp_filter.needMoreResponses())
                         done=true;
@@ -478,7 +478,7 @@ public class GroupRequest implements RspCollector, Command {
         }
 
         try {
-            if(log.isTraceEnabled()) log.trace(new StringBuffer("sending request (id=").append(req_id).append(')'));
+            if(log.isTraceEnabled()) log.trace(new StringBuilder("sending request (id=").append(req_id).append(')'));
             if(corr != null) {
                 List<Address> tmp=new Vector<Address>(members);
                 corr.sendRequest(req_id, tmp, request_msg, rsp_mode == GET_NONE? null : this, use_anycasting);
