@@ -1,15 +1,14 @@
 package org.jgroups.protocols;
 
 import org.jgroups.*;
+import org.jgroups.protocols.pbcast.JoinRsp;
 import org.jgroups.stack.Protocol;
 import org.jgroups.util.Promise;
 import org.jgroups.util.TimeScheduler;
 
 import java.util.*;
-import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.TimeoutException;
 
 
 /**
@@ -28,16 +27,16 @@ import java.util.concurrent.TimeoutException;
  * <li>num_ping_requests - the number of GET_MBRS_REQ messages to be sent (min=1), distributed over timeout ms
  * </ul>
  * @author Bela Ban
- * @version $Id: Discovery.java,v 1.32.2.1 2007/11/20 08:37:24 belaban Exp $
+ * @version $Id: Discovery.java,v 1.32.2.3 2008/01/22 10:01:19 belaban Exp $
  */
 public abstract class Discovery extends Protocol {
     final Vector<Address>	members=new Vector<Address>(11);
-    Address					local_addr=null;
-    String					group_addr=null;
-    long					timeout=3000;
-    int						num_initial_members=2;
-    boolean					is_server=false;
-    TimeScheduler			timer=null;
+    Address			local_addr=null;
+    String			group_addr=null;
+    long			timeout=3000;
+    int				num_initial_members=2;
+    boolean			is_server=false;
+    TimeScheduler		timer=null;
 
     /** Number of GET_MBRS_REQ messages to be sent (min=1), distributed over timeout ms */
     int                     num_ping_requests=2;
@@ -143,8 +142,8 @@ public abstract class Discovery extends Protocol {
         }
 
         if(!props.isEmpty()) {
-            StringBuffer sb=new StringBuffer();
-            for(Enumeration e=props.propertyNames(); e.hasMoreElements();) {
+            StringBuilder sb=new StringBuilder();
+            for(Enumeration<?> e=props.propertyNames(); e.hasMoreElements();) {
                 sb.append(e.nextElement().toString());
                 if(e.hasMoreElements()) {
                     sb.append(", ");
@@ -174,7 +173,7 @@ public abstract class Discovery extends Protocol {
      * until 'num_initial_members' have been retrieved
      * @return List<PingRsp>
      */
-    public List<PingRsp> findInitialMembers(Promise promise) {
+    public List<PingRsp> findInitialMembers(Promise<JoinRsp> promise) {
         num_discovery_requests++;
 
         final Responses rsps=new Responses(num_initial_members, promise);
@@ -187,7 +186,7 @@ public abstract class Discovery extends Protocol {
             return rsps.get(timeout);
         }
         catch(Exception e) {
-            return null;
+            return new LinkedList<PingRsp>();           
         }
         finally {
         	sender.stop();
@@ -318,7 +317,7 @@ public abstract class Discovery extends Protocol {
 
         case Event.FIND_INITIAL_MBRS:   // sent by GMS layer, pass up a GET_MBRS_OK event
             // sends the GET_MBRS_REQ to all members, waits 'timeout' ms or until 'num_initial_members' have been retrieved
-            return findInitialMembers((Promise)evt.getArg());
+            return findInitialMembers((Promise<JoinRsp>)evt.getArg());
 
         case Event.TMP_VIEW:
         case Event.VIEW_CHANGE:
@@ -371,8 +370,8 @@ public abstract class Discovery extends Protocol {
     
 
     class PingSenderTask {
-        private double		interval;
-		private Future		senderFuture;
+        private double          interval;
+        private Future<?>	senderFuture;
 
         public PingSenderTask(long timeout, int num_requests) {
             interval=timeout / (double)num_requests;
@@ -404,13 +403,13 @@ public abstract class Discovery extends Protocol {
 
 
     private static class Responses {
-        final Promise       promise;
+        final Promise<JoinRsp>       promise;
         final List<PingRsp> ping_rsps=new LinkedList<PingRsp>();
         final int           num_expected_rsps;
 
-        public Responses(int num_expected_rsps, Promise promise) {
+        public Responses(int num_expected_rsps, Promise<JoinRsp> promise) {
             this.num_expected_rsps=num_expected_rsps;
-            this.promise=promise != null? promise : new Promise();
+            this.promise=promise != null? promise : new Promise<JoinRsp>();
         }
 
         public void addResponse(PingRsp rsp) {
@@ -429,7 +428,7 @@ public abstract class Discovery extends Protocol {
             }
         }
 
-        public List<PingRsp> get(long timeout) throws InterruptedException, ExecutionException, TimeoutException {
+        public List<PingRsp> get(long timeout) throws InterruptedException {
             long start_time=System.currentTimeMillis(), time_to_wait=timeout;
 
             promise.getLock().lock();
