@@ -39,11 +39,11 @@ import java.util.concurrent.locks.ReentrantLock;
  * <li>{@link #stop()}: subclasses <em>must</em> call super.stop() after they deinitialized themselves
  * <li>{@link #destroy()}
  * </ul>
- * The create() or start() method has to create a local address.<br>
+ * The create() or start() method has to create a local address.<br
  * The {@link #receive(Address, Address, byte[], int, int)} method must
  * be called by subclasses when a unicast or multicast message has been received.
  * @author Bela Ban
- * @version $Id: TP.java,v 1.160.2.1 2007/11/28 09:19:59 belaban Exp $
+ * @version $Id: TP.java,v 1.160.2.12 2008/02/25 08:00:23 belaban Exp $
  */
 public abstract class TP extends Protocol {
 
@@ -86,10 +86,9 @@ public abstract class TP extends Protocol {
     int				port_range=1; // 27-6-2003 bgooren, Only try one port by default
 
     /** The members of this group (updated when a member joins or leaves) */
-    final protected Vector<Address> members=new Vector<Address>(11);
-
-    protected View            view=null;
+    final protected HashSet<Address>   members=new HashSet<Address>(11);
 
+    protected View                     view=null;
 
     final ExposedByteArrayInputStream  in_stream=new ExposedByteArrayInputStream(new byte[]{'0'});
     final DataInputStream              dis=new DataInputStream(in_stream);
@@ -159,8 +158,17 @@ public abstract class TP extends Protocol {
      * JDK 5's java.util.concurrent package */
     String oob_thread_pool_rejection_policy="Run";
 
+    public Executor getOOBThreadPool() {
+        return oob_thread_pool;
+    }
 
-/** ================================== Regular thread pool ============================== */
+    public void setOOBThreadPool(Executor oob_thread_pool) {
+        if(this.oob_thread_pool != null) {
+            shutdownThreadPool(oob_thread_pool);
+        }
+        this.oob_thread_pool=oob_thread_pool;
+    }
+    /** ================================== Regular thread pool ============================== */
 
     /** The thread pool which handles unmarshalling, version checks and dispatching of regular messages */
     Executor thread_pool;
@@ -182,6 +190,17 @@ public abstract class TP extends Protocol {
      * JDK 5's java.util.concurrent package */
     String thread_pool_rejection_policy="Run";
 
+    public Executor getDefaultThreadPool() {
+        return thread_pool;
+    }
+
+    public void setDefaultThreadPool(Executor thread_pool) {
+        if(this.thread_pool != null)
+            shutdownThreadPool(this.thread_pool);
+        this.thread_pool=thread_pool;
+    } /** =============================== End of regular thread pool ============================= */
+
+
 
     /** If set it will be added to <tt>local_addr</tt>. Used to implement
      * for example transport independent addresses */
@@ -211,6 +230,15 @@ public abstract class TP extends Protocol {
     String diagnostics_addr="224.0.0.75";
     int    diagnostics_port=7500;
 
+    /** If this transport is shared, identifies all the transport instances which are to be shared */
+    String singleton_name=null;
+
+    /** If singleton_name is enabled, this map is used to de-multiplex incoming messages according to their
+     * cluster names (attached to the message by the transport anyway). The values are the next protocols above
+     * the transports.
+     */
+    private final ConcurrentMap<String,Protocol> up_prots=new ConcurrentHashMap<String,Protocol>();
+
     TpHeader header;
     final String name=getName();
 
@@ -294,9 +322,9 @@ public abstract class TP extends Protocol {
     public void setLoopback(boolean b) {loopback=b;}
     public boolean isUseIncomingPacketHandler() {return use_incoming_packet_handler;}
 
-
-
-
+    public ConcurrentMap<String,Protocol> getUpProtocols() {
+        return up_prots;
+    }
 
     public int getOOBMinPoolSize() {
         return oob_thread_pool instanceof ThreadPoolExecutor? ((ThreadPoolExecutor)oob_thread_pool).getCorePoolSize() : 0;
@@ -432,8 +460,8 @@ public abstract class TP extends Protocol {
     public abstract void postUnmarshallingList(Message msg, Address dest, boolean multicast);
 
 
-    private StringBuffer _getInfo() {
-        StringBuffer sb=new StringBuffer();
+    private StringBuilder _getInfo() {
+        StringBuilder sb=new StringBuilder();
         sb.append(local_addr).append(" (").append(channel_name).append(") ").append("\n");
         sb.append("local_addr=").append(local_addr).append("\n");
         sb.append("group_name=").append(channel_name).append("\n");
@@ -448,7 +476,7 @@ public abstract class TP extends Protocol {
         try {
             StringTokenizer tok=new StringTokenizer(request);
             String req=tok.nextToken();
-            StringBuffer info=new StringBuffer("n/a");
+            StringBuilder info=new StringBuilder("n/a");
             if(req.trim().toLowerCase().startsWith("query")) {
                 ArrayList<String> l=new ArrayList<String>(tok.countTokens());
                 while(tok.hasMoreTokens())
@@ -460,7 +488,7 @@ public abstract class TP extends Protocol {
                     Channel ch=stack.getChannel();
                     if(ch != null) {
                         Map m=ch.dumpStats();
-                        StringBuffer sb=new StringBuffer();
+                        StringBuilder sb=new StringBuilder();
                         sb.append("stats:\n");
                         for(Iterator it=m.entrySet().iterator(); it.hasNext();) {
                             sb.append(it.next()).append("\n");
@@ -472,9 +500,15 @@ public abstract class TP extends Protocol {
                     String p=stack.printProtocolSpecAsXML();
                     info.append("\nprops:\n").append(p);
                 }
+                if(l.contains("info")) {
+                    Map<String, Object> tmp=stack.getChannel().getInfo();
+                    info.append("INFO:\n");
+                    for(Map.Entry<String,Object> entry: tmp.entrySet()) {
+                        info.append(entry.getKey()).append(": ").append(entry.getValue()).append("\n");
+                    }
+                }
             }
 
-
             byte[] diag_rsp=info.toString().getBytes();
             if(log.isDebugEnabled())
                 log.debug("sending diag response to " + sender);
@@ -503,7 +537,7 @@ public abstract class TP extends Protocol {
         if(bind_addr != null) {
             Map<String,Object> m=new HashMap<String,Object>(1);
             m.put("bind_addr", bind_addr);
-            up_prot.up(new Event(Event.CONFIG, m));
+            up(new Event(Event.CONFIG, m));
         }
         
         HashMap <String,ThreadNamingPattern> map = new HashMap<String,ThreadNamingPattern>();
@@ -512,7 +546,6 @@ public abstract class TP extends Protocol {
     }
 
 
-
     /**
      * Creates the unicast and multicast sockets and starts the unicast and multicast receiver threads
      */
@@ -534,30 +567,36 @@ public abstract class TP extends Protocol {
 
 
         // ========================================== OOB thread pool ==============================
-        if(oob_thread_pool_enabled) { // create a ThreadPoolExecutor for the unmarshaller thread pool
-            if(oob_thread_pool_queue_enabled)
-                oob_thread_pool_queue=new LinkedBlockingQueue<Runnable>(oob_thread_pool_queue_max_size);
-            else
-                oob_thread_pool_queue=new SynchronousQueue<Runnable>();
-            oob_thread_pool=createThreadPool(oob_thread_pool_min_threads, oob_thread_pool_max_threads, oob_thread_pool_keep_alive_time,
-                                             oob_thread_pool_rejection_policy, oob_thread_pool_queue, "OOB");
-        }
-        else { // otherwise use the caller's thread to unmarshal the byte buffer into a message
-            oob_thread_pool=new DirectExecutor();
+        // create a ThreadPoolExecutor for the unmarshaller thread pool
+        if(oob_thread_pool == null) { // only create if not yet set (e.g. by a user)
+            if(oob_thread_pool_enabled) {
+                if(oob_thread_pool_queue_enabled)
+                    oob_thread_pool_queue=new LinkedBlockingQueue<Runnable>(oob_thread_pool_queue_max_size);
+                else
+                    oob_thread_pool_queue=new SynchronousQueue<Runnable>();
+                oob_thread_pool=createThreadPool(oob_thread_pool_min_threads, oob_thread_pool_max_threads, oob_thread_pool_keep_alive_time,
+                                                 oob_thread_pool_rejection_policy, oob_thread_pool_queue, "OOB");
+            }
+            else { // otherwise use the caller's thread to unmarshal the byte buffer into a message
+                oob_thread_pool=new DirectExecutor();
+            }
         }
 
 
         // ====================================== Regular thread pool ===========================
-        if(thread_pool_enabled) { // create a ThreadPoolExecutor for the unmarshaller thread pool
-            if(thread_pool_queue_enabled)
-                thread_pool_queue=new LinkedBlockingQueue<Runnable>(thread_pool_queue_max_size);
-            else
-                thread_pool_queue=new SynchronousQueue<Runnable>();
-            thread_pool=createThreadPool(thread_pool_min_threads, thread_pool_max_threads, thread_pool_keep_alive_time,
-                                         thread_pool_rejection_policy, thread_pool_queue, "Incoming");
-        }
-        else { // otherwise use the caller's thread to unmarshal the byte buffer into a message
-            thread_pool=new DirectExecutor();
+        // create a ThreadPoolExecutor for the unmarshaller thread pool
+        if(thread_pool == null) { // only create if not yet set (e.g.by a user)
+            if(thread_pool_enabled) {
+                if(thread_pool_queue_enabled)
+                    thread_pool_queue=new LinkedBlockingQueue<Runnable>(thread_pool_queue_max_size);
+                else
+                    thread_pool_queue=new SynchronousQueue<Runnable>();
+                thread_pool=createThreadPool(thread_pool_min_threads, thread_pool_max_threads, thread_pool_keep_alive_time,
+                                             thread_pool_rejection_policy, thread_pool_queue, "Incoming");
+            }
+            else { // otherwise use the caller's thread to unmarshal the byte buffer into a message
+                thread_pool=new DirectExecutor();
+            }
         }
 
 
@@ -572,7 +611,7 @@ public abstract class TP extends Protocol {
         }
 
         thread_naming_pattern.setAddress(local_addr);
-        up_prot.up(new Event(Event.SET_LOCAL_ADDRESS, local_addr));
+        sendUpLocalAddressEvent();
     }
 
 
@@ -594,11 +633,13 @@ public abstract class TP extends Protocol {
         // 3. Stop the thread pools
 
         if(oob_thread_pool instanceof ThreadPoolExecutor) {
-            shutdownThreadPool((ThreadPoolExecutor)oob_thread_pool);
+            shutdownThreadPool(oob_thread_pool);
+            oob_thread_pool=null;
         }
 
         if(thread_pool instanceof ThreadPoolExecutor) {
-            shutdownThreadPool((ThreadPoolExecutor)thread_pool);
+            shutdownThreadPool(thread_pool);
+            thread_pool=null;
         }
     }
 
@@ -618,13 +659,8 @@ public abstract class TP extends Protocol {
         try {
             bind_addr=Util.getBindAddress(props);
         }
-        catch(UnknownHostException unknown) {
-            log.fatal("failed getting bind_addr", unknown);
-            return false;
-        }
-        catch(SocketException ex) {
-            log.fatal("failed getting bind_addr", ex);
-            return false;
+        catch(IOException unknown) {
+            throw new IllegalArgumentException("failed determining bind address", unknown);
         }
 
         str=props.getProperty("use_local_host");
@@ -915,6 +951,12 @@ public abstract class TP extends Protocol {
             props.remove("diagnostics_port");
         }
 
+        str=props.getProperty(Global.SINGLETON_NAME);
+        if(str != null) {
+            singleton_name=str;
+            props.remove(Global.SINGLETON_NAME);
+        }
+
         return true;
     }
 
@@ -928,12 +970,20 @@ public abstract class TP extends Protocol {
     public Object up(Event evt) {
         switch(evt.getType()) {
         case Event.CONFIG:
-            up_prot.up(evt);
+            if(singleton_name != null)
+                passToAllUpProtocols(evt);
+            else
+                up_prot.up(evt);
             if(log.isDebugEnabled()) log.debug("received CONFIG event: " + evt.getArg());
             handleConfigEvent((Map<String,Object>)evt.getArg());
             return null;
         }
-        return up_prot.up(evt);
+        if(singleton_name != null) {
+            passToAllUpProtocols(evt);
+            return null;
+        }
+        else
+            return up_prot.up(evt);
     }
 
     /**
@@ -951,7 +1001,7 @@ public abstract class TP extends Protocol {
         if(header != null) {
             // added patch by Roland Kurmann (March 20 2003)
             // msg.putHeader(name, new TpHeader(channel_name));
-            msg.putHeader(name, header);
+            msg.putHeaderIfAbsent(name, header);
         }
 
         setSourceAddress(msg); // very important !! listToBuffer() will fail with a null src address !!
@@ -969,14 +1019,14 @@ public abstract class TP extends Protocol {
             // we *have* to make a copy, or else up_prot.up() might remove headers from msg which will then *not*
             // be available for marshalling further down (when sending the message)
             final Message copy=msg.copy();
-            if(log.isTraceEnabled()) log.trace(new StringBuffer("looping back message ").append(copy));
+            if(log.isTraceEnabled()) log.trace(new StringBuilder("looping back message ").append(copy));
             // up_prot.up(new Event(Event.MSG, copy));
 
             // changed to fix http://jira.jboss.com/jira/browse/JGRP-506
             Executor pool=msg.isFlagSet(Message.OOB)? oob_thread_pool : thread_pool;
             pool.execute(new Runnable() {
                 public void run() {
-                    up_prot.up(new Event(Event.MSG, copy));
+                    passMessageUp(copy, false);
                 }
             });
 
@@ -1019,6 +1069,78 @@ public abstract class TP extends Protocol {
             msg.setSrc(local_addr);
     }
 
+
+    private void passMessageUp(Message msg, boolean perform_cluster_name_matching) {
+        TpHeader hdr=(TpHeader)msg.getHeader(name); // replaced removeHeader() with getHeader()
+        if(hdr == null) {
+            if(channel_name == null) {
+                Event evt=new Event(Event.MSG, msg);
+                if(singleton_name != null) {
+                    passMessageToAll(evt);
+                }
+                else {
+                    up_prot.up(evt);
+                }
+            }
+            else {
+                if(log.isErrorEnabled())
+                    log.error(new StringBuilder("message does not have a transport header, msg is ").append(msg).
+                            append(", headers are ").append(msg.printHeaders()).append(", will be discarded"));
+            }
+            return;
+        }
+
+        String ch_name=hdr.channel_name;
+        if(singleton_name != null) {
+            Protocol tmp_prot=up_prots.get(ch_name);
+            if(tmp_prot != null) {
+                Event evt=new Event(Event.MSG, msg);
+                if(log.isTraceEnabled()) {
+                    StringBuilder sb=new StringBuilder("message is ").append(msg).append(", headers are ").append(msg.printHeaders());
+                    log.trace(sb);
+                }
+                tmp_prot.up(evt);
+            }
+            else {
+                // we discard messages for a group we don't have. If we had a scenario with channel C1 and A,B on it,
+                // and channel C2 and only A on it (asymmetric setup), then C2 would always log warnings that B was
+                // not found (Jan 25 2008 (bela))
+                // if(log.isWarnEnabled())
+                   // log.warn(new StringBuilder("discarded message from group \"").append(ch_name).
+                     //       append("\" (our groups are ").append(up_prots.keySet()).append("). Sender was ").append(msg.getSrc()));
+            }
+        }
+        else {
+            // Discard if message's group name is not the same as our group name
+            if(perform_cluster_name_matching && channel_name != null && !channel_name.equals(ch_name)) {
+                if(log.isWarnEnabled())
+                    log.warn(new StringBuilder("discarded message from different group \"").append(ch_name).
+                            append("\" (our group is \"").append(channel_name).append("\"). Sender was ").append(msg.getSrc()));
+            }
+            else {
+                Event evt=new Event(Event.MSG, msg);
+                if(log.isTraceEnabled()) {
+                    StringBuilder sb=new StringBuilder("message is ").append(msg).append(", headers are ").append(msg.printHeaders());
+                    log.trace(sb);
+                }
+                up_prot.up(evt);
+            }
+        }
+    }
+
+    private void passMessageToAll(Event evt) {
+        for(Protocol tmp_prot: up_prots.values()) {
+            try {
+                tmp_prot.up(evt);
+            }
+            catch(Exception ex) {
+                if(log.isErrorEnabled())
+                    log.error("failure passing message up: message is " + evt.getArg(), ex);
+            }
+        }
+    }
+
+
     /**
      * Subclasses must call this method when a unicast or multicast message has been received.
      * Declared final so subclasses cannot override this method.
@@ -1068,7 +1190,7 @@ public abstract class TP extends Protocol {
         }
         catch(Throwable t) {
             if(log.isErrorEnabled())
-                log.error(new StringBuffer("failed handling data from ").append(sender), t);
+                log.error(new StringBuilder("failed handling data from ").append(sender), t);
         }
     }
 
@@ -1113,7 +1235,7 @@ public abstract class TP extends Protocol {
                 }
                 if(Version.isBinaryCompatible(version) == false) {
                     if(log.isWarnEnabled()) {
-                        StringBuffer sb=new StringBuffer();
+                        StringBuilder sb=new StringBuilder();
                         sb.append("packet from ").append(sender).append(" has different version (").append(Version.print(version));
                         sb.append(") from ours (").append(Version.printVersion()).append("). ");
                         if(discard_incompatible_packets)
@@ -1163,42 +1285,11 @@ public abstract class TP extends Protocol {
 
 
     private void handleIncomingMessage(Message msg) {
-        Event      evt;
-        TpHeader   hdr;
-
         if(stats) {
             num_msgs_received++;
             num_bytes_received+=msg.getLength();
         }
-
-        evt=new Event(Event.MSG, msg);
-        if(log.isTraceEnabled()) {
-            StringBuffer sb=new StringBuffer("message is ").append(msg).append(", headers are ").append(msg.printHeaders());
-            log.trace(sb);
-        }
-
-        hdr=(TpHeader)msg.getHeader(name); // replaced removeHeader() with getHeader()
-        if(hdr != null) {
-
-            /* Discard all messages destined for a channel with a different name */
-            String ch_name=hdr.channel_name;
-
-            // Discard if message's group name is not the same as our group name unless the
-            // message is a diagnosis message (special group name DIAG_GROUP)
-            if(channel_name != null && !channel_name.equals(ch_name)) {
-                if(log.isWarnEnabled())
-                    log.warn(new StringBuffer("discarded message from different group \"").append(ch_name).
-                            append("\" (our group is \"").append(channel_name).append("\"). Sender was ").append(msg.getSrc()));
-                return;
-            }
-        }
-        else {
-            if(log.isTraceEnabled())
-                log.trace(new StringBuffer("message does not have a transport header, msg is ").append(msg).
-                          append(", headers are ").append(msg.printHeaders()).append(", will be discarded"));
-            return;
-        }
-        up_prot.up(evt);
+        passMessageUp(msg, true);
     }
 
 
@@ -1320,27 +1411,39 @@ public abstract class TP extends Protocol {
             synchronized(members) {
                 view=(View)evt.getArg();
                 members.clear();
-                Vector<Address> tmpvec=view.getMembers();
-                members.addAll(tmpvec);
+
+                if(singleton_name == null) {
+                    Vector<Address> tmpvec=view.getMembers();
+                    members.addAll(tmpvec);
+                }
+                else {
+                    for(Protocol prot: up_prots.values()) {
+                        if(prot instanceof ProtocolAdapter) {
+                            ProtocolAdapter ad=(ProtocolAdapter)prot;
+                            List<Address> tmp=ad.getMembers();
+                            members.addAll(tmp);
+                        }
+                    }
+                }
             }
             break;
 
-        case Event.CONNECT:
-        case Event.CONNECT_WITH_STATE_TRANSFER:    
-            channel_name=(String)evt.getArg();
-            header=new TpHeader(channel_name);            
-            thread_naming_pattern.setClusterName(channel_name);
-            setThreadNames();                               
-            return null;
+            case Event.CONNECT:
+            case Event.CONNECT_WITH_STATE_TRANSFER:
+                channel_name=(String)evt.getArg();
+                header=new TpHeader(channel_name);
+                thread_naming_pattern.setClusterName(channel_name);
+                setThreadNames();
+                return null;
 
-        case Event.DISCONNECT:
-            unsetThreadNames();
-            break;
+            case Event.DISCONNECT:
+                unsetThreadNames();
+                break;
 
-        case Event.CONFIG:
-            if(log.isDebugEnabled()) log.debug("received CONFIG event: " + evt.getArg());
-            handleConfigEvent((Map<String,Object>)evt.getArg());
-            break;
+            case Event.CONFIG:
+                if(log.isDebugEnabled()) log.debug("received CONFIG event: " + evt.getArg());
+                handleConfigEvent((Map<String,Object>)evt.getArg());
+                break;
         }
         return null;
     }
@@ -1380,8 +1483,8 @@ public abstract class TP extends Protocol {
     }
 
 
-    protected Executor createThreadPool(int min_threads, int max_threads, long keep_alive_time, String rejection_policy,
-                                        BlockingQueue<Runnable> queue, final String thread_name) {
+    protected ExecutorService createThreadPool(int min_threads, int max_threads, long keep_alive_time, String rejection_policy,
+                                               BlockingQueue<Runnable> queue, final String thread_name) {
 
         ThreadPoolExecutor pool=new ThreadPoolExecutor(min_threads, max_threads, keep_alive_time, TimeUnit.MILLISECONDS, queue);
         pool.setThreadFactory(ProtocolStack.newThreadFactory(thread_naming_pattern, pool_thread_group, thread_name, false));
@@ -1401,16 +1504,45 @@ public abstract class TP extends Protocol {
     }
 
 
-    private static void shutdownThreadPool(ThreadPoolExecutor thread_pool) {
-        thread_pool.shutdownNow();
-        try {
-            thread_pool.awaitTermination(Global.THREADPOOL_SHUTDOWN_WAIT_TIME, TimeUnit.MILLISECONDS);
-        }
-        catch(InterruptedException e) {
+    private static void shutdownThreadPool(Executor thread_pool) {
+        if(thread_pool instanceof ExecutorService) {
+            ExecutorService service=(ExecutorService)thread_pool;
+            service.shutdownNow();
+            try {
+                service.awaitTermination(Global.THREADPOOL_SHUTDOWN_WAIT_TIME, TimeUnit.MILLISECONDS);
+            }
+            catch(InterruptedException e) {
+            }
         }
     }
 
 
+    protected void passToAllUpProtocols(Event evt) {
+        for(Protocol prot: up_prots.values()) {
+            try {
+                prot.up(evt);
+            }
+            catch(Exception e) {
+                if(log.isErrorEnabled())
+                    log.error("failed passing up event " + evt, e);
+            }
+        }
+    }
+
+    public void sendUpLocalAddressEvent() {
+        if(up_prot != null)
+            up(new Event(Event.SET_LOCAL_ADDRESS, local_addr));
+        else {
+            for(Map.Entry<String,Protocol> entry: up_prots.entrySet()) {
+                String tmp=entry.getKey();
+                if(tmp.startsWith(Global.DUMMY))
+                    continue;
+                Protocol prot=entry.getValue();
+                prot.up(new Event(Event.SET_LOCAL_ADDRESS, local_addr));
+            }
+        }
+    }
+
     /* ----------------------------- End of Private Methods ---------------------------------------- */
 
 
@@ -1453,7 +1585,7 @@ public abstract class TP extends Protocol {
                 }
                 if(Version.isBinaryCompatible(version) == false) {
                     if(log.isWarnEnabled()) {
-                        StringBuffer sb=new StringBuffer();
+                        StringBuilder sb=new StringBuilder();
                         sb.append("packet from ").append(sender).append(" has different version (").append(Version.print(version));
                         sb.append(") from ours (").append(Version.printVersion()).append("). ");
                         if(discard_incompatible_packets)
@@ -1502,40 +1634,8 @@ public abstract class TP extends Protocol {
                 return; // drop message that was already looped back and delivered
             }
 
-            TpHeader hdr=(TpHeader)msg.getHeader(name); // replaced removeHeader() with getHeader()
-            if(hdr != null) {
-                String ch_name=hdr.channel_name;
-
-                // Discard if message's group name is not the same as our group name
-                if(channel_name != null && !channel_name.equals(ch_name)) {
-                    if(log.isWarnEnabled())
-                        log.warn(new StringBuffer("discarded message from different group \"").append(ch_name).
-                                append("\" (our group is \"").append(channel_name).append("\"). Sender was ").append(msg.getSrc()));
-                    return;
-                }
-            }
-            else {
-                if(channel_name == null) {
-                    ;
-                }
-                else {
-                    if(log.isTraceEnabled())
-                        log.trace(new StringBuffer("message does not have a transport header, msg is ").append(msg).
-                                append(", headers are ").append(msg.printHeaders()).append(", will be discarded"));
-                    return;
-                }
-            }
-
-            Event evt=new Event(Event.MSG, msg);
-            if(log.isTraceEnabled()) {
-                StringBuffer sb=new StringBuffer("message is ").append(msg).append(", headers are ").append(msg.printHeaders());
-                log.trace(sb);
-            }
-
-            up_prot.up(evt);
+            passMessageUp(msg, true);
         }
-
-
     }
 
 
@@ -1867,4 +1967,47 @@ public abstract class TP extends Protocol {
             }
         }
     }
+
+    public static class ProtocolAdapter extends Protocol {
+        final String cluster_name;
+        final String transport_name;
+        final TpHeader header;
+        final List<Address> members=new ArrayList<Address>();
+
+        public ProtocolAdapter(String cluster_name, String transport_name, Protocol up, Protocol down) {
+            this.cluster_name=cluster_name;
+            this.transport_name=transport_name;
+            this.up_prot=up;
+            this.down_prot=down;
+            this.header=new TpHeader(cluster_name);
+}
+
+        public List<Address> getMembers() {
+            return Collections.unmodifiableList(members);
+        }
+
+        public Object down(Event evt) {
+            switch(evt.getType()) {
+                case Event.MSG:
+                    Message msg=(Message)evt.getArg();
+                    msg.putHeader(transport_name, header);
+                    break;
+                case Event.VIEW_CHANGE:
+                    View view=(View)evt.getArg();
+                    Vector<Address> tmp=view.getMembers();
+                    members.clear();
+                    members.addAll(tmp);
+                    break;
+            }
+            return down_prot.down(evt);
+        }
+
+        public String getName() {
+            return null;
+        }
+
+        public String toString() {
+            return cluster_name + " (" + transport_name + ")";
+        }
+    }
 }
