@@ -1,4 +1,4 @@
-// $Id: UNICAST.java,v 1.91 2007/11/02 13:18:50 belaban Exp $
+// $Id: UNICAST.java,v 1.91.2.2 2008/01/22 10:01:19 belaban Exp $
 
 package org.jgroups.protocols;
 
@@ -51,6 +51,15 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
     /** whether to loop back messages sent to self (will be removed in the future, default=false) */
     private boolean          loopback=false;
 
+    /**
+     * By default, we release the lock on the sender in up() after the up() method call passed up the stack returns.
+     * However, with eager_lock_release enabled (default), we release the lock as soon as the application calls
+     * Channel.down() <em>within</em> the receive() callback. This leads to issues as the one described in
+     * http://jira.jboss.com/jira/browse/JGRP-656. Note that ordering is <em>still correct </em>, but messages from self
+     * might get delivered concurrently. This can be turned off by setting eager_lock_release to false.
+     */
+    private boolean eager_lock_release=true;
+
     /** A list of members who left, used to determine when to prevent sending messages to left mbrs */
     private final BoundedList<Address> previous_members=new BoundedList<Address>(50);
     /** Contains all members that were enabled for unicasts by Event.ENABLE_UNICAST_TO */
@@ -204,6 +213,12 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
             props.remove("loopback");
         }
 
+        str=props.getProperty("eager_lock_release");
+        if(str != null) {
+            eager_lock_release=Boolean.valueOf(str).booleanValue();
+            props.remove("eager_lock_release");
+        }
+
         if(!props.isEmpty()) {
             log.error("these properties are not recognized: " + props);
             return false;
@@ -327,7 +342,7 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
                         }
                         msg.putHeader(name, hdr);
                         if(log.isTraceEnabled())
-                            log.trace(new StringBuffer().append(local_addr).append(" --> DATA(").append(dst).append(": #").
+                            log.trace(new StringBuilder().append(local_addr).append(" --> DATA(").append(dst).append(": #").
                                     append(seqno));
                         entry.sent_msgs.add(seqno, msg);  // add *including* UnicastHeader, adds to retransmitter
                         entry.sent_msgs_seqno++;
@@ -517,7 +532,7 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
      */
     private boolean handleDataReceived(Address sender, long seqno, Message msg) {
         if(log.isTraceEnabled())
-            log.trace(new StringBuffer().append(local_addr).append(" <-- DATA(").append(sender).append(": #").append(seqno));
+            log.trace(new StringBuilder().append(local_addr).append(" <-- DATA(").append(sender).append(": #").append(seqno));
 
         if(previous_members.contains(sender)) {
             // we don't want to see messages from departed members
@@ -570,7 +585,8 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
         ReentrantLock lock=win.getLock();
         lock.lock(); // we don't block on entry any more (http://jira.jboss.com/jira/browse/JGRP-485)
         try {
-            locks.put(Thread.currentThread(), lock);
+            if(eager_lock_release)
+                locks.put(Thread.currentThread(), lock);
             while((m=win.remove()) != null) {
                 // discard OOB msg as it has already been delivered (http://jira.jboss.com/jira/browse/JGRP-377)
                 if(m.isFlagSet(Message.OOB)) {
@@ -580,7 +596,8 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
             }
         }
         finally {
-            locks.remove(Thread.currentThread());
+            if(eager_lock_release)
+                locks.remove(Thread.currentThread());
             if(lock.isHeldByCurrentThread())
                 lock.unlock();
         }
@@ -594,7 +611,7 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
         AckSenderWindow win;
 
         if(log.isTraceEnabled())
-            log.trace(new StringBuffer().append(local_addr).append(" <-- ACK(").append(sender).
+            log.trace(new StringBuilder().append(local_addr).append(" <-- ACK(").append(sender).
                       append(": #").append(seqno).append(')'));
         synchronized(connections) {
             entry=connections.get(sender);
@@ -613,7 +630,7 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
         ack.setFlag(Message.OOB);
         ack.putHeader(name, new UnicastHeader(UnicastHeader.ACK, seqno));
         if(log.isTraceEnabled())
-            log.trace(new StringBuffer().append(local_addr).append(" --> ACK(").append(dst).
+            log.trace(new StringBuilder().append(local_addr).append(" --> ACK(").append(dst).
                       append(": #").append(seqno).append(')'));
         down_prot.down(new Event(Event.MSG, ack));
         num_acks_sent++;
