@@ -60,10 +60,7 @@ public class FLUSH extends Protocol {
     private final List<Address> flushMembers;
 
     @GuardedBy("sharedLock")
-    private final Map<Address, Digest> flushCompletedMap;
-
-    @GuardedBy("sharedLock")
-    private final Set<Address> stopFlushOkSet;
+    private final Map<Address, Digest> flushCompletedMap;    
 
     @GuardedBy("sharedLock")
     private final Set<Address> suspected;
@@ -84,15 +81,15 @@ public class FLUSH extends Protocol {
      * <code>isBlockingFlushDown</code>
      */
     private long timeout = 8000;
+     
+    private long start_flush_timeout = 2500;
     
-    /**
-     * Default timeout for a group member to wait for <code>Channel#startFlush</code> to return
-     * 
-     */
-    private long start_flush_timeout = 8000;
+    private long retry_timeout = 3000;
 
     private boolean enable_reconciliation = true;
     
+    private int flush_retry_count = 4;
+    
     private final AtomicInteger viewCounter = new AtomicInteger(0);   
     
     private volatile boolean allowMessagesToPassUp = false;
@@ -119,8 +116,7 @@ public class FLUSH extends Protocol {
     public FLUSH(){
         super();
         currentView = new View(new ViewId(), new Vector<Address>());      
-        flushCompletedMap = new HashMap<Address, Digest>();
-        stopFlushOkSet = new TreeSet<Address>();
+        flushCompletedMap = new HashMap<Address, Digest>();        
         flushMembers = new ArrayList<Address>();
         suspected = new TreeSet<Address>();
     }
@@ -133,7 +129,9 @@ public class FLUSH extends Protocol {
         super.setProperties(props);
 
         timeout = Util.parseLong(props, "timeout", timeout);
-        start_flush_timeout = Util.parseLong(props, "start_flush_timeout", start_flush_timeout);
+        flush_retry_count = Util.parseInt(props, "flush_retry_count", flush_retry_count);
+        start_flush_timeout = Util.parseLong(props, "start_flush_timeout", start_flush_timeout);        
+        retry_timeout = Util.parseLong(props, "retry_timeout", retry_timeout);
         enable_reconciliation = Util.parseBoolean(props,
                                                   "enable_reconciliation",
                                                   enable_reconciliation);
@@ -166,8 +164,7 @@ public class FLUSH extends Protocol {
     public void stop() {
         synchronized(sharedLock){
             currentView = new View(new ViewId(), new Vector<Address>());
-            flushCompletedMap.clear();            
-            stopFlushOkSet.clear();
+            flushCompletedMap.clear();                       
             flushMembers.clear();
             suspected.clear();
             flushCoordinator = null;
@@ -192,12 +189,8 @@ public class FLUSH extends Protocol {
         return startFlush(new Event(Event.SUSPEND));
     }
     
-    private boolean startFlush(Event evt){
-        int numberOfAttempts = 2;
-        synchronized (sharedLock) {
-            numberOfAttempts = (currentView != null) ? currentView.size() : numberOfAttempts;
-        }
-        return startFlush(evt, numberOfAttempts);
+    private boolean startFlush(Event evt){        
+        return startFlush(evt, flush_retry_count);
     }
     
     private boolean startFlush(Event evt, int numberOfAttempts) {
@@ -207,7 +200,7 @@ public class FLUSH extends Protocol {
             if(log.isDebugEnabled())
                 log.debug("Received " + evt + " at " + localAddress + ". Running FLUSH...");           
 
-            onSuspend((View) evt.getArg());
+            onSuspend((List<Address>) evt.getArg());
             try{
                 Boolean r = flush_promise.getResultWithTimeout(start_flush_timeout);
                 successfulFlush = r.booleanValue();
@@ -221,13 +214,7 @@ public class FLUSH extends Protocol {
         }
 
         if(!successfulFlush && numberOfAttempts > 0){                  
-            //we sleep for at most start_flush_timeout and
-            //at least until current flush ends 
-            long start_time = System.currentTimeMillis(), backofftime = start_flush_timeout;
-            while (backofftime > 0 && flushInProgress.get()) {
-                Util.sleep(Util.random(5)*1000);
-                backofftime = start_flush_timeout - (System.currentTimeMillis() - start_time);
-            }      
+            waitForFlushCompletion(retry_timeout);      
             if(log.isDebugEnabled()){               
                 log.debug("Retrying FLUSH at " + localAddress
                           + ", "
@@ -272,16 +259,24 @@ public class FLUSH extends Protocol {
         case Event.CONNECT:
         case Event.CONNECT_WITH_STATE_TRANSFER:   
             if(sentBlock.compareAndSet(false, true)){                
-                sendBlockUpToChannel();
-                sentUnblock.set(false);
+                sendBlockUpToChannel();              
             }
+
+            Object result=down_prot.down(evt);
+            if(result instanceof Throwable) {
+                sentBlock.set(false); // set the var back to its original state if we cannot connect successfully
+            }
+            return result;
+            
+        case Event.DISCONNECT:
+            waitForFlushCompletion(retry_timeout);
             break;
 
         case Event.SUSPEND:
             return startFlush(evt);
 
         case Event.RESUME:
-            onResume();
+            onResume(evt);
             return null;
         }
         return down_prot.down(evt);
@@ -333,7 +328,16 @@ public class FLUSH extends Protocol {
                     case FlushHeader.FLUSH_BYPASS:
                         return up_prot.up(evt);                     
                     case FlushHeader.START_FLUSH:
-                        handleStartFlush(msg, fh);
+                        Collection<Address> fp=fh.flushParticipants;
+                        boolean amIParticipant = fp != null && fp.contains(localAddress);
+                        if(amIParticipant){
+                            handleStartFlush(msg, fh);
+                        }         
+                        else{ 
+                            if (log.isDebugEnabled())                        
+                                log.debug("Received START_FLUSH at " + localAddress
+                                      + " but I am not flush participant, not responding");                                            
+                        }
                         break;
                     case FlushHeader.FLUSH_RECONCILE:
                         handleFlushReconcile(msg, fh);
@@ -361,6 +365,11 @@ public class FLUSH extends Protocol {
                 // for processing of application messages after we join, 
                 // lets wait for STOP_FLUSH to complete
                 // before we start allowing message up.
+                Address dest=msg.getDest();
+                if(dest != null && !dest.isMulticastAddress()) {
+                    return up_prot.up(evt); // allow unicasts to pass, virtual synchrony olny applies to multicasts
+                }
+
                 if(!allowMessagesToPassUp)
                     return null;
             }
@@ -419,13 +428,22 @@ public class FLUSH extends Protocol {
             return startFlush(evt);
 
         case Event.RESUME:
-            onResume();
+            onResume(evt);
             return null;
 
         }
 
         return up_prot.up(evt);
     }
+    
+    private boolean waitForFlushCompletion(long timeout){
+        long start_time = System.currentTimeMillis(), backofftime = timeout;
+        while (backofftime > 0 && flushInProgress.get()) {
+            Util.sleep(100);
+            backofftime = timeout - (System.currentTimeMillis() - start_time);            
+        }
+        return backofftime < 0; 
+    }
 
     private void onFlushReconcileOK(Message msg) {
         if(log.isDebugEnabled())
@@ -466,30 +484,33 @@ public class FLUSH extends Protocol {
         down_prot.down(new Event(Event.MSG, reconcileOk));
     }
 
-    private void handleStartFlush(Message msg, FlushHeader fh) {        
-        boolean proceed = false;
+    private void handleStartFlush(Message msg, FlushHeader fh) {                
         Address flushRequester = msg.getSrc();
         Address abortFlushCoordinator = null;
         Address proceedFlushCoordinator = null;        
         
-        synchronized (sharedLock) {
-            proceed = flushInProgress.compareAndSet(false, true);                     
+        boolean proceed = flushInProgress.compareAndSet(false, true);
+        synchronized (sharedLock) {                               
             if(proceed){
                 flushCoordinator = flushRequester;
-            }else{                                                                                             
-                if(flushRequester.compareTo(flushCoordinator) < 0){                                                           
-                    abortFlushCoordinator = flushCoordinator;
-                    flushCoordinator = flushRequester;
-                    proceedFlushCoordinator = flushRequester;                    
-                }else if(flushRequester.compareTo(flushCoordinator) > 0){                                        
-                    abortFlushCoordinator = flushRequester;
-                    proceedFlushCoordinator = flushCoordinator;
-                }else{                    
-                    if(log.isDebugEnabled()){
-                        log.debug("Rejecting flush at " + localAddress + ", previous flush has to finish first");
-                    } 
-                    abortFlushCoordinator = flushRequester;
-                    proceedFlushCoordinator = flushCoordinator;
+            }else{
+                if(flushCoordinator != null){
+                    if(flushRequester.compareTo(flushCoordinator) < 0){                                                           
+                        abortFlushCoordinator = flushCoordinator;
+                        flushCoordinator = flushRequester;
+                        proceedFlushCoordinator = flushRequester;                    
+                    }else if(flushRequester.compareTo(flushCoordinator) > 0){                                        
+                        abortFlushCoordinator = flushRequester;
+                        proceedFlushCoordinator = flushCoordinator;
+                    }else{                    
+                        if(log.isDebugEnabled()){
+                            log.debug("Rejecting flush at " + localAddress + ", previous flush has to finish first");
+                        } 
+                        abortFlushCoordinator = flushRequester;
+                        proceedFlushCoordinator = flushCoordinator;
+                    }
+                }else{
+                    return;
                 }
             }
         }
@@ -523,6 +544,12 @@ public class FLUSH extends Protocol {
 
     private void sendBlockUpToChannel() {
         up_prot.up(new Event(Event.BLOCK));
+        sentUnblock.set(false);
+    }
+    
+    private void sendUnBlockUpToChannel() {
+        sentBlock.set(false);
+        up_prot.up(new Event(Event.UNBLOCK));       
     }
 
     private boolean isCurrentFlushMessage(FlushHeader fh) {
@@ -545,7 +572,7 @@ public class FLUSH extends Protocol {
         synchronized(sharedLock){            
             suspected.retainAll(view.getMembers());
             currentView = view;
-            coordinatorLeft = flushCoordinator != null && !view.containsMember(flushCoordinator);            
+            coordinatorLeft = view.getMembers().size()>0 && !view.containsMember(view.getCreator());            
         }      
         if(log.isDebugEnabled())
             log.debug("Installing view at  " + localAddress + " view is " + view);
@@ -561,12 +588,9 @@ public class FLUSH extends Protocol {
                 averageFlushDuration = totalTimeInFlush / (double) numberOfFlushes;
             }
         }
-        
-        boolean amISurvivingMember = false;               
-        synchronized(sharedLock){            
-            amISurvivingMember = currentView.containsMember(localAddress);
-            flushCompletedMap.clear();          
-            stopFlushOkSet.clear();
+                            
+        synchronized(sharedLock){                     
+            flushCompletedMap.clear();                     
             flushMembers.clear();
             suspected.clear();
             flushCoordinator = null;
@@ -582,21 +606,20 @@ public class FLUSH extends Protocol {
             blockMutex.notifyAll();
         }
                 
-        if(amISurvivingMember && sentUnblock.compareAndSet(false,true)){
+        if(sentUnblock.compareAndSet(false,true)){
             //ensures that we do not repeat unblock event            
-            up_prot.up(new Event(Event.UNBLOCK));        
-            sentBlock.set(false);
+            sendUnBlockUpToChannel();                  
         }                
         flushInProgress.set(false);
     }
 
-    private void onSuspend(View view) {
+    private void onSuspend(List<Address> members) {
         Message msg = null;
         Collection<Address> participantsInFlush = null;
         synchronized(sharedLock){
             // start FLUSH only on group members that we need to flush
-            if(view != null){
-                participantsInFlush = new ArrayList<Address>(view.getMembers());
+            if(members != null){
+                participantsInFlush = members;
                 participantsInFlush.retainAll(currentView.getMembers());
             }else{
                 participantsInFlush = new ArrayList<Address>(currentView.getMembers());
@@ -617,62 +640,68 @@ public class FLUSH extends Protocol {
         }
     }
 
-    private void onResume() {
+    private void onResume(Event evt) {
+        List<Address> members = (List<Address>) evt.getArg();
         long viewID = currentViewId();
-        Message msg = new Message(null, localAddress, null);    
-        //Cannot be OOB since START_FLUSH is not OOB
-        //we have to FIFO order two subsequent flushes       
-        msg.putHeader(getName(), new FlushHeader(FlushHeader.STOP_FLUSH, viewID));
-        down_prot.down(new Event(Event.MSG, msg));
-        if(log.isDebugEnabled())
-            log.debug("Received RESUME at " + localAddress + ", sent STOP_FLUSH to all");
+        if(members == null || members.isEmpty()){
+            Message msg = new Message(null, localAddress, null);    
+            //Cannot be OOB since START_FLUSH is not OOB
+            //we have to FIFO order two subsequent flushes       
+            msg.putHeader(getName(), new FlushHeader(FlushHeader.STOP_FLUSH, viewID));
+            down_prot.down(new Event(Event.MSG, msg));
+            if(log.isDebugEnabled())
+                log.debug("Received RESUME at " + localAddress + ", sent STOP_FLUSH to all");
+        }else{
+            for (Address address : members) {
+                Message msg = new Message(address, localAddress, null);    
+                //Cannot be OOB since START_FLUSH is not OOB
+                //we have to FIFO order two subsequent flushes       
+                msg.putHeader(getName(), new FlushHeader(FlushHeader.STOP_FLUSH, viewID));
+                down_prot.down(new Event(Event.MSG, msg));
+                if(log.isDebugEnabled())
+                    log.debug("Received RESUME at " + localAddress + ", sent STOP_FLUSH to " + address);
+            }
+        }
     }
 
-    private void onStartFlush(Address flushStarter, FlushHeader fh) {             
-        synchronized(blockMutex){
-            isBlockingFlushDown = true;
-        }       
+    private void onStartFlush(Address flushStarter, FlushHeader fh) {                           
         if(stats){
             startFlushTime = System.currentTimeMillis();
             numberOfFlushes += 1;
-        }
-        boolean amIParticipant = false;
+        }        
         synchronized(sharedLock){
             flushCoordinator = flushStarter;
             flushMembers.clear();
             if(fh.flushParticipants != null){
                 flushMembers.addAll(fh.flushParticipants);
             }
-            flushMembers.removeAll(suspected);
-            amIParticipant = flushMembers.contains(localAddress);
+            flushMembers.removeAll(suspected);            
         }
         
-        if(amIParticipant && sentBlock.compareAndSet(false, true)){
+        if(sentBlock.compareAndSet(false, true)){
             //ensures that we do not repeat block event
             //and that we do not send block event to non participants            
-            sendBlockUpToChannel();
-            sentUnblock.set(false);
+            sendBlockUpToChannel();            
+            synchronized(blockMutex){
+                isBlockingFlushDown = true;
+            }
         }
         else{
             if(log.isDebugEnabled())
-                log.debug("Received START_FLUSH at " + localAddress + " but not sending UNBLOCK up");
-        }
-        if(amIParticipant){                      
+                log.debug("Received START_FLUSH at " + localAddress + " but not sending BLOCK up");
+        }                        
             
-            Digest digest = (Digest) down_prot.down(new Event(Event.GET_DIGEST));
-            FlushHeader fhr = new FlushHeader(FlushHeader.FLUSH_COMPLETED, fh.viewID);
-            fhr.addDigest(digest);            
-            
-            Message msg = new Message(flushStarter);
-            msg.putHeader(getName(), fhr);
-            down_prot.down(new Event(Event.MSG, msg));
-            if(log.isDebugEnabled())
-                log.debug("Received START_FLUSH at " + localAddress + " responded with FLUSH_COMPLETED");
-        }
-        else{
-            if(log.isDebugEnabled())
-                log.debug("Received START_FLUSH at " + localAddress + " but I am not participant, not responding");
-        }
+        Digest digest = (Digest) down_prot.down(new Event(Event.GET_DIGEST));
+        FlushHeader fhr = new FlushHeader(FlushHeader.FLUSH_COMPLETED, fh.viewID);
+        fhr.addDigest(digest);            
+        
+        Message msg = new Message(flushStarter);
+        msg.putHeader(getName(), fhr);
+        down_prot.down(new Event(Event.MSG, msg));
+        if(log.isDebugEnabled())
+            log.debug("Received START_FLUSH at " + localAddress
+                      + " responded with FLUSH_COMPLETED");
+              
     }
     
     private void onFlushCompleted(Address address, Digest digest) {
@@ -826,6 +855,18 @@ public class FLUSH extends Protocol {
             if(flushView != null){
                 this.flushParticipants = new ArrayList<Address>(flushView);
             }
+        }             
+
+        @Override
+        public int size() {
+            int retval=Global.BYTE_SIZE; // type            
+            retval+=Global.LONG_SIZE; // viewID            
+            retval+= Util.size(flushParticipants);                      
+            retval+=Global.BYTE_SIZE; // presence for digest
+            if(digest != null){
+                retval += digest.serializedSize();
+            }
+            return retval;
         }
 
         public void addDigest(Digest digest) {
@@ -866,7 +907,7 @@ public class FLUSH extends Protocol {
         public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
             type = in.readByte();
             viewID = in.readLong();
-            flushParticipants = (Collection) in.readObject();
+            flushParticipants = (Collection<Address>) in.readObject();
             digest = (Digest) in.readObject();
         }
 
