@@ -30,7 +30,7 @@ import java.util.concurrent.locks.ReentrantLock;
  * to everyone instead of the requester by setting use_mcast_xmit to true.
  *
  * @author Bela Ban
- * @version $Id: NAKACK.java,v 1.170 2007/11/02 16:48:10 belaban Exp $
+ * @version $Id: NAKACK.java,v 1.170.2.4 2008/02/05 16:00:43 belaban Exp $
  */
 public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand, NakReceiverWindow.Listener {
     private long[]              retransmit_timeouts={600, 1200, 2400, 4800}; // time(s) to wait before requesting retransmission
@@ -81,6 +81,15 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
      */
     private boolean discard_delivered_msgs=false;
 
+    /**
+     * By default, we release the lock on the sender in up() after the up() method call passed up the stack returns.
+     * However, with eager_lock_release enabled (default), we release the lock as soon as the application calls
+     * Channel.down() <em>within</em> the receive() callback. This leads to issues as the one described in
+     * http://jira.jboss.com/jira/browse/JGRP-656. Note that ordering is <em>still correct </em>, but messages from self
+     * might get delivered concurrently. This can be turned off by setting eager_lock_release to false.
+     */
+    private boolean eager_lock_release=true;
+
     /** If value is > 0, the retransmit buffer is bounded: only the max_xmit_buf_size latest messages are kept,
      * older ones are discarded when the buffer size is exceeded. A value <= 0 means unbounded buffers
      */
@@ -392,6 +401,12 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
             props.remove("max_rebroadcast_timeout");
         }
 
+        str=props.getProperty("eager_lock_release");
+        if(str != null) {
+            eager_lock_release=Boolean.valueOf(str).booleanValue();
+            props.remove("eager_lock_release");
+        }
+
         if(xmit_from_random_member) {
             if(discard_delivered_msgs) {
                 discard_delivered_msgs=false;
@@ -829,7 +844,8 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
         ReentrantLock lock=win.getLock();
         lock.lock();
         try {
-            locks.put(Thread.currentThread(), lock);
+            if(eager_lock_release)
+                locks.put(Thread.currentThread(), lock);
             while((msg_to_deliver=win.remove()) != null) {
 
                 // discard OOB msg as it has already been delivered (http://jira.jboss.com/jira/browse/JGRP-379)
@@ -843,7 +859,8 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
             }
         }
         finally {
-            locks.remove(Thread.currentThread());
+            if(eager_lock_release)
+                locks.remove(Thread.currentThread());
             if(lock.isHeldByCurrentThread())
                 lock.unlock();
         }
@@ -894,7 +911,7 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
         NakReceiverWindow win=xmit_table.get(original_sender);
         if(win == null) {
             if(log.isErrorEnabled()) {
-                StringBuffer sb=new StringBuffer();
+                StringBuilder sb=new StringBuilder();
                 sb.append("(requester=").append(xmit_requester).append(", local_addr=").append(this.local_addr);
                 sb.append(") ").append(original_sender).append(" not found in retransmission table: ").append(printMessages());
                 if(print_stability_history_on_failed_xmit) {
@@ -907,15 +924,15 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
         for(long i=first_seqno; i <= last_seqno; i++) {
             msg=win.get(i);
             if(msg == null || msg == NakReceiverWindow.NULL_MSG) {
-                if(log.isErrorEnabled() && !local_addr.equals(xmit_requester)) {
-                    StringBuffer sb=new StringBuffer();
+                if(log.isWarnEnabled() && !local_addr.equals(xmit_requester)) {
+                    StringBuilder sb=new StringBuilder();
                     sb.append("(requester=").append(xmit_requester).append(", local_addr=").append(this.local_addr);
                     sb.append(") message ").append(original_sender).append("::").append(i);
                     sb.append(" not found in retransmission table of ").append(printMessages());
                     if(print_stability_history_on_failed_xmit) {
                         sb.append(" (stability history:\n").append(printStabilityHistory());
                     }
-                    log.error(sb);
+                    log.warn(sb);
                 }
                 continue;
             }
@@ -1120,6 +1137,11 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
         for(Iterator<Address> it=xmit_table.keySet().iterator(); it.hasNext();) {
             Address sender=it.next();
             if(!new_members.contains(sender)) {
+                if(local_addr != null && local_addr.equals(sender)) {
+                    if(log.isErrorEnabled())
+                        log.error("will not remove myself (" + sender + ") from xmit_table, received incorrect new membership of " + new_members);
+                    continue;
+                }
                 win=xmit_table.get(sender);
                 win.reset();
                 if(log.isDebugEnabled()) {
@@ -1173,7 +1195,21 @@ public class NAKACK extends Protocol implements Retransmitter.RetransmitCommand,
             return;
         }
 
-        clear();
+        if(local_addr != null && digest.contains(local_addr)) {
+            clear();
+        }
+        else {
+            // remove all but local_addr (if not null)
+            for(Iterator<Address> it=xmit_table.keySet().iterator(); it.hasNext();) {
+                Address key=it.next();
+                if(local_addr != null && local_addr.equals(key)) {
+                    ;
+                }
+                else {
+                    it.remove();
+                }
+            }
+        }
 
         Address sender;
         Digest.Entry val;
