@@ -27,7 +27,7 @@ import java.util.concurrent.locks.ReentrantLock;
  * New: when <code>max_bytes</code> is exceeded (unless disabled by setting it to 0),
  * a STABLE task will be started (unless it is already running). Design in docs/design/STABLE.txt
  * @author Bela Ban
- * @version $Id: STABLE.java,v 1.85 2007/09/19 15:55:56 belaban Exp $
+ * @version $Id: STABLE.java,v 1.85.2.1 2008/01/22 10:01:28 belaban Exp $
  */
 public class STABLE extends Protocol {
     private Address               local_addr=null;
@@ -588,7 +588,7 @@ public class STABLE extends Protocol {
          }
 
          if(log.isTraceEnabled())
-             log.trace(new StringBuffer("received stability msg from ").append(sender).append(": ").append(stable_digest.printHighestDeliveredSeqnos()));
+             log.trace(new StringBuilder("received stability msg from ").append(sender).append(": ").append(stable_digest.printHighestDeliveredSeqnos()));
          stopStabilityTask();
 
         lock.lock();
