@@ -9,6 +9,7 @@ import org.jgroups.protocols.PingHeader;
 import org.jgroups.protocols.UdpHeader;
 import org.jgroups.protocols.pbcast.NakAckHeader;
 import org.jgroups.stack.IpAddress;
+import org.jgroups.stack.Protocol;
 
 import javax.management.MBeanServer;
 import javax.management.MBeanServerFactory;
@@ -27,7 +28,7 @@ import java.util.*;
 /**
  * Collection of various utility routines that can not be assigned to other classes.
  * @author Bela Ban
- * @version $Id: Util.java,v 1.137 2007/10/01 07:16:18 belaban Exp $
+ * @version $Id: Util.java,v 1.137.2.2 2008/01/21 13:53:26 belaban Exp $
  */
 public class Util {
 
@@ -2026,11 +2027,7 @@ public class Util {
         InetAddress retval=null, bind_addr_host=null;
 
         if(bind_addr != null) {
-            try {
-                bind_addr_host=InetAddress.getByName(bind_addr);
-            }
-            catch(UnknownHostException e) {
-            }
+            bind_addr_host=InetAddress.getByName(bind_addr);
         }
 
         if(bind_interface != null) {
@@ -2288,6 +2285,14 @@ public class Util {
     }
 
 
+    public static String getProperty(Protocol prot, String prop_name) {
+        if(prot == null)
+            return null;
+        String name=prot.getProperties().getProperty(prop_name);
+        return name == null? name : name.trim();
+    }
+
+
     /*
     public static void main(String[] args) {
 	DatagramSocket sock;
