@@ -26,7 +26,6 @@ import org.jgroups.Event;
 import org.jgroups.ExtendedReceiverAdapter;
 import org.jgroups.GetStateEvent;
 import org.jgroups.JChannel;
-import org.jgroups.JChannelFactory;
 import org.jgroups.Message;
 import org.jgroups.SetStateEvent;
 import org.jgroups.UnblockEvent;
@@ -40,7 +39,7 @@ import org.jgroups.util.Util;
  * configured to use FLUSH
  * 
  * @author Bela Ban
- * @version $Id: FlushTest.java,v 1.58 2007/11/09 01:59:40 vlada Exp $
+ * @version $Id: FlushTest.java,v 1.58.2.2 2008/02/04 03:13:20 vlada Exp $
  */
 public class FlushTest extends ChannelTestBase {
     private JChannel c1, c2;
@@ -146,6 +145,30 @@ public class FlushTest extends ChannelTestBase {
         // now send unicast, this might block as described in the case
         c2.send(unicast_msg);
     }
+    
+    /**
+     * Tests http://jira.jboss.com/jira/browse/JGRP-661
+     * @throws Exception 
+     */
+    public void testPartialFlush() throws Exception {
+        c1 = createChannel();
+        c1.setReceiver(new SimpleReplier(c1, true));
+        c1.connect("test");
+    
+        c2 = createChannel();
+        c2.setReceiver(new SimpleReplier(c2, false));
+        c2.connect("test");
+       
+        List<Address> members = new ArrayList<Address>();
+        members.add(c2.getLocalAddress());       
+        boolean flushedOk = c2.startFlush(members, false);
+        
+        assertTrue("Partial flush worked", flushedOk);
+        
+        c2.stopFlush(members);
+               
+
+    }
 
     /**
      * Tests emition of block/unblock/get|set state events in both mux and bare
@@ -445,116 +468,7 @@ public class FlushTest extends ChannelTestBase {
             }
         }
     }
-
-    private void checkEventSequence(FlushTestReceiver receiver, boolean isMuxUsed) {
-        List<Object> events = receiver.getEvents();
-        String eventString = "[" + receiver.getName()
-                             + "|"
-                             + receiver.getLocalAddress()
-                             + ",events:"
-                             + events;
-        log.info(eventString);        
-        assertNotNull(events);
-        int size = events.size();
-        for(int i = 0;i < size;i++){
-            Object event = events.get(i);
-            if(event instanceof BlockEvent){
-                if(i + 1 < size){
-                    Object ev = events.get(i + 1);
-                    if(isMuxUsed){
-                        assertTrue("After Block should be View or Unblock but it is " + ev.getClass() + ",events= " + eventString,
-                                   ev instanceof View || ev instanceof UnblockEvent);
-                    }else{
-                        assertTrue("After Block should be View but it is " + ev.getClass() + ",events= " + eventString,
-                                   ev instanceof View);
-                    }
-                }
-                if(i > 0){
-                    Object ev = events.get(i - 1);
-                    assertTrue("Before Block should be Unblock but it is " + ev.getClass() + ",events= " + eventString,
-                               ev instanceof UnblockEvent);
-                }
-            }
-            else if(event instanceof View){
-                if(i + 1 < size){
-                    Object ev = events.get(i + 1);
-                    assertTrue("After View should be Unblock but it is " + ev.getClass() + ",events= " + eventString,
-                               ev instanceof UnblockEvent);
-                }
-                Object ev = events.get(i - 1);
-                assertTrue("Before View should be Block but it is " + ev.getClass() + ",events= " + eventString,
-                           ev instanceof BlockEvent);
-            }
-            else if(event instanceof UnblockEvent){
-                if(i + 1 < size){
-                    Object ev = events.get(i + 1);
-                    assertTrue("After UnBlock should be Block but it is " + ev.getClass() + ",events= " + eventString,
-                               ev instanceof BlockEvent);
-                }
-
-                Object ev = events.get(i - 1);
-                if(isMuxUsed){
-                    assertTrue("Before UnBlock should be View or Block but it is " + ev.getClass() + ",events= " + eventString,
-                               ev instanceof View || ev instanceof BlockEvent);
-                }else{
-                    assertTrue("Before UnBlock should be View but it is " + ev.getClass() + ",events= " + eventString,
-                               ev instanceof View);
-                }
-            }
-        }       
-    }
-
-    private void checkEventStateTransferSequence(FlushTestReceiver receiver) {
-        
-        List<Object> events = receiver.getEvents();
-        String eventString = "[" + receiver.getName() + ",events:" + events;
-        log.info(eventString);        
-        assertNotNull(events);
-        int size = events.size();
-        for(int i = 0;i < size;i++){
-            Object event = events.get(i);
-            if(event instanceof BlockEvent){
-                if(i + 1 < size){
-                    Object o = events.get(i + 1);
-                    assertTrue("After Block should be state|unblock|view, but it is " + o.getClass() + ",events= "+ eventString,
-                               o instanceof SetStateEvent || o instanceof GetStateEvent
-                                       || o instanceof UnblockEvent
-                                       || o instanceof View);
-                }
-                if(i > 0){
-                    Object o = events.get(i - 1);
-                    assertTrue("Before Block should be state or Unblock , but it is " + o.getClass() + ",events= " + eventString, 
-                               o instanceof UnblockEvent);
-                }
-            }
-            else if(event instanceof SetStateEvent || event instanceof GetStateEvent){
-                if(i + 1 < size){
-                    Object o = events.get(i + 1);
-                    assertTrue("After state should be Unblock , but it is " + o.getClass() + ",events= " + eventString,
-                               o instanceof UnblockEvent);
-                }
-                Object o = events.get(i - 1);
-                assertTrue("Before state should be Block or View , but it is " + o.getClass() + ",events= " + eventString,
-                           o instanceof BlockEvent || events.get(i - 1) instanceof View);
-            }
-            else if(event instanceof UnblockEvent){
-                if(i + 1 < size){
-                    Object o = events.get(i + 1);
-                    assertTrue("After UnBlock should be Block , but it is " + o.getClass() + ",events= " + eventString,
-                               o instanceof BlockEvent);
-                }
-                if(i > 0){
-                    Object o = events.get(i - 1);
-                    assertTrue("Before UnBlock should be block|state|view , but it is " + o.getClass() + ",events= " + eventString,
-                               o instanceof SetStateEvent || o instanceof GetStateEvent
-                                       || o instanceof BlockEvent
-                                       || o instanceof View);
-                }
-            }
-
-        }        
-    }
-
+    
     protected JChannel createChannel() throws ChannelException {
         JChannel ret = new JChannel(CHANNEL_CONFIG);
         ret.setOpt(Channel.BLOCK, Boolean.TRUE);
